using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.IGControls;
using Infragistics.Win.UltraWinSchedule;
using nsDayView = Infragistics.Win.UltraWinSchedule.DayView;
using nsMonthViewMulti = Infragistics.Win.UltraWinSchedule.MonthViewMulti;
using nsUIAction = Infragistics.Win.UltraWinSchedule.TimelineView.UIActionNotification;
using Infragistics.Win.AppStyling;
using Infragistics.Win.AppStyling.Runtime;
using nsTimeline = Infragistics.Win.UltraWinSchedule.TimelineView;
using System.Diagnostics;
using System.Text.RegularExpressions;
using nsEditors = Infragistics.Win.UltraWinEditors;

namespace TimelineViewTesting
{
    #region frmMain class
    public partial class frmMain : Form
    {
        public frmMain()
        {
            this.InitializeComponent();
            this.InitializeUI();

        }

        private void InitializeUI()
        {
            DateTime zero = DateTime.Today;
            this.timeLineView.CalendarInfo.MinDate = zero;
            this.timeLineView.AdditionalIntervals.Clear();
            this.timeLineView.PrimaryInterval = new VideoFrameTimeInterval( this.timeLineView, zero );

            List<VideoTrack> tracks = new List<VideoTrack>();
            VideoTrack track1 = new VideoTrack(100, 120);
            track1.DoubleClick += new VideoTrack.DoubleClickHandler(track1_DoubleClick);
            tracks.Add( track1 );
            tracks.Add( new VideoTrack(120, 420) );
            tracks.Add( new VideoTrack(105, 620) );
            tracks.Add( new VideoTrack(102, 450) );
            tracks.Add( new VideoTrack(101, 510) );

            this.timeLineView.CreationFilter = new VideoFrameCreationFilter( zero, 25, Color.Red, Color.Green, tracks );
            this.timeLineView.CalendarInfo.BeforeDisplayAppointmentDialog += new DisplayAppointmentDialogEventHandler(CalendarInfo_BeforeDisplayAppointmentDialog);

            this.timeLineView.MouseDoubleClick += new MouseEventHandler(timeLineView_MouseDoubleClick);
            this.propertyGrid1.SelectedObject = new VideoFrameInfo( this.timeLineView, zero );
        }

        void track1_DoubleClick(object sender, frmMain.TrackEventArgs e)
        {
            bool stop = true;
        }

        void timeLineView_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            UltraTimelineView control = sender as UltraTimelineView;
            VideoFrameInfo info = this.propertyGrid1.SelectedObject as VideoFrameInfo;
            if ( info != null )
            {
                long? frame = info.FrameFromPoint( e.Location );
                if ( frame.HasValue )
                    MessageBox.Show( string.Format("Frame = {0}", frame.Value) );
            }
        }


        void CalendarInfo_BeforeDisplayAppointmentDialog(object sender, DisplayAppointmentDialogEventArgs e)
        {
            e.Cancel = true;
        }

        #region VideoFrameCreationFilter class
        public class VideoFrameCreationFilter : IUIElementCreationFilter
        {
            private DateTime zero = DateTime.Today;
            private int frameRate = 25;
            private Color evenMinuteBorderColor = Color.Red;
            private Color evenSecondBorderColor = Color.Green;
            List<VideoFrameElement> elementCache = new List<VideoFrameElement>(10);
            List<VideoFrameElement> newElements = new List<VideoFrameElement>(10);
            List<VideoTrack> tracks = null;
            private int trackHeight = 32;

            public VideoFrameCreationFilter(
                DateTime zero,
                int frameRate,
                Color evenMinuteBorderColor,
                Color evenSecondBorderColor,
                List<VideoTrack> tracks )
            {
                this.zero = zero;
                this.frameRate = frameRate;
                this.evenMinuteBorderColor = evenMinuteBorderColor;
                this.evenSecondBorderColor = evenSecondBorderColor;
                this.tracks = tracks;
            }

            #region IUIElementCreationFilter

            void IUIElementCreationFilter.AfterCreateChildElements(UIElement parent)
            {
            }

            bool IUIElementCreationFilter.BeforeCreateChildElements(UIElement parent)
            {
                Infragistics.Win.UltraWinSchedule.TimelineView.ActivityAreaUIElement activityAreaElement =
                    parent as Infragistics.Win.UltraWinSchedule.TimelineView.ActivityAreaUIElement;

                if ( activityAreaElement != null )
                {
                    activityAreaElement.ChildElements.Clear();
                    this.AddFrameElements( activityAreaElement );
                    this.AddTrackElements( activityAreaElement );
                    return true;
                }

                return false;
            }

            #endregion IUIElementCreationFilter
            
            private void AddTrackElements( Infragistics.Win.UltraWinSchedule.TimelineView.ActivityAreaUIElement activityAreaElement )
            {
                const int SPACE = 3;

                Rectangle rectInsideBorders = activityAreaElement.RectInsideBorders;
                UltraTimelineView control = activityAreaElement.Control as UltraTimelineView;
                int colWidth = control.ColumnWidthResolved;                
                DateTime firstFrame = control.VisibleDateTimeRange.StartDateTime;                
                int slots = (int)(Math.Ceiling( (double)rectInsideBorders.Width / (double)colWidth ));                

                long startFrame = (long)(firstFrame.Subtract( this.zero ).TotalMinutes);
                long endFrame = startFrame + slots;

                DateTime thisFrame = firstFrame;

                int top = rectInsideBorders.Top + SPACE;

                for ( int i = 0, count = this.tracks.Count; i < count; i ++ )
                {
                    VideoTrack track = this.tracks[i];
                    if ( track.StartFrame > endFrame ||
                         track.EndFrame < startFrame )
                        continue;

                    Rectangle rect = rectInsideBorders;
                    rect.Y += i == 0 ? SPACE : (i * (this.trackHeight + SPACE));
                    rect.Height = this.trackHeight;
                    rect.X = (int)((track.StartFrame - startFrame) * colWidth) + (SPACE * 2);
                    rect.Width = (int)((track.EndFrame - track.StartFrame) * colWidth) - (SPACE * 3);

                    VideoTrackElement trackElement = new VideoTrackElement( activityAreaElement, track );
                    trackElement.Rect = rect;
                    activityAreaElement.ChildElements.Add( trackElement );

                    top += (this.trackHeight + SPACE);
                }
            }

            private void AddFrameElements( Infragistics.Win.UltraWinSchedule.TimelineView.ActivityAreaUIElement activityAreaElement )
            {
                Rectangle rectInsideBorders = activityAreaElement.RectInsideBorders;
                UltraTimelineView control = activityAreaElement.Control as UltraTimelineView;
                int colWidth = control.ColumnWidthResolved;                
                DateTime firstFrame = control.VisibleDateTimeRange.StartDateTime;                
                Rectangle rect = activityAreaElement.RectInsideBorders;
                int slots = (int)(Math.Ceiling( (double)rectInsideBorders.Width / (double)colWidth ));                
                Rectangle workRect = rect;
                workRect.Width = colWidth - 1;

                long frame = (long)(firstFrame.Subtract( this.zero ).TotalMinutes);
                DateTime thisFrame = firstFrame;

                for ( int i = 0; i < slots; i ++ )
                {
                    thisFrame = thisFrame.AddMinutes( i );
                    bool isEvenMinute = (frame % (this.frameRate * 60)) == 0;
                    bool isEvenSecond = (frame % this.frameRate) == 0;
                    UIElementBorderStyle borderStyle = isEvenSecond ? UIElementBorderStyle.TwoColor : UIElementBorderStyle.Dotted;
                    Color borderColor =
                        isEvenMinute ? this.evenMinuteBorderColor :
                        isEvenSecond ? this.evenSecondBorderColor :
                        Color.DarkGray;

                    VideoFrameElement frameElement =
                        VideoFrameCreationFilter.GetFrameElement(
                        activityAreaElement,
                        this.elementCache,
                        frame + 1,
                        borderStyle,
                        borderColor,
                        Color.White );

                    frameElement.Rect = workRect;
                    activityAreaElement.ChildElements.Add( frameElement );

                    workRect.X += colWidth;
                    frame++;

                    this.newElements.Add( frameElement );
                }

                this.elementCache.AddRange( this.newElements );

                this.newElements.Clear();
            }

            private static VideoFrameElement GetFrameElement(
                Infragistics.Win.UltraWinSchedule.TimelineView.ActivityAreaUIElement activityAreaElement,
                List<VideoFrameElement> elementCache,
                long frame,
                UIElementBorderStyle borderStyle,
                Color borderColor,
                Color backColor )
            {
                if ( elementCache == null || elementCache.Count == 0 )
                    return new VideoFrameElement( activityAreaElement, frame, borderStyle, borderColor, backColor );
                else
                {
                    VideoFrameElement retVal = elementCache[0];
                    retVal.Initialize( frame, borderStyle, borderColor, backColor );
                    elementCache.RemoveAt(0);
                    return retVal;
                }
            }

            #region VideoFrameElement
            /// <summary>
            /// Simple UIElement which replaces a time slot element.
            /// </summary>
            public class VideoFrameElement : UIElement
            {
                #region Members
                private long frame = 0;
                private UIElementBorderStyle borderStyle = UIElementBorderStyle.Solid;
                private Color borderColor = Color.Black;
                private Color backColor = Color.White;
                #endregion Members

                #region Constructor
                /// <summary>
                /// Creates a new instance.
                /// </summary>
                public VideoFrameElement(
                    UIElement parentElement,
                    long frame,
                    UIElementBorderStyle borderStyle,
                    Color borderColor,
                    Color backColor ) :
                    base( parentElement )
                {
                    this.Initialize( frame, borderStyle, borderColor, backColor );
                }

                /// <summary>
                /// Recycles this instance.
                /// </summary>
                public void Initialize( long frame, UIElementBorderStyle borderStyle, Color borderColor, Color backColor )
                {
                    this.frame = frame;
                    this.borderStyle = borderStyle;
                    this.borderColor = borderColor;
                    this.backColor = backColor;
                }
                #endregion Constructor

                #region BorderSides
                /// <summary>
                /// Only draw the left border.
                /// </summary>
                public override Border3DSide BorderSides
                {
                    get { return Border3DSide.Left; }
                    set { throw new NotSupportedException(); }
                }
                #endregion BorderSides

                #region BorderStyle
                /// <summary>
                /// Returns the border style for this element.
                /// </summary>
                public override UIElementBorderStyle BorderStyle
                {
                    get { return this.borderStyle; }                    
                    set { throw new NotSupportedException(); }
                }
                #endregion BorderStyle

                #region InitAppearance
                /// <summary>
                /// Initializes the appearance data for this element.
                /// </summary>
                protected override void InitAppearance(ref AppearanceData appearance, ref AppearancePropFlags requestedProps)
                {
                    #region BorderColor
                    if ( (requestedProps & AppearancePropFlags.BorderColor) == AppearancePropFlags.BorderColor &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor) == false )
                    {
                        appearance.BorderColor = this.borderColor;
                        requestedProps &= ~AppearancePropFlags.BorderColor;
                    }
                    #endregion BorderColor

                    #region BorderColor2
                    if ( (requestedProps & AppearancePropFlags.BorderColor2) == AppearancePropFlags.BorderColor2 &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor2) == false )
                    {
                        appearance.BorderColor2 = Color.FromArgb(128, this.borderColor);
                        requestedProps &= ~AppearancePropFlags.BorderColor2;
                    }
                    #endregion BorderColor2

                    #region BackColor
                    if ( (requestedProps & AppearancePropFlags.BackColor) == AppearancePropFlags.BackColor &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BackColor) == false )
                    {
                        appearance.BackColor = this.backColor;
                        requestedProps &= ~AppearancePropFlags.BackColor;
                    }
                    #endregion BackColor
                }
                #endregion InitAppearance

                #region WantsInputNotification
                //  Uncomment if you want this element to not be interactive.
                ///// <summary>
                ///// Returns false so that this element does notrespond to user interaction.
                ///// </summary>
                //protected override bool WantsInputNotification(UIElementInputType inputType, Point point)
                //{
                //    return false;
                //}
                #endregion WantsInputNotification

                #region Frame
                /// <summary>
                /// Returns the frame number this element represents.
                /// </summary>
                public long Frame
                {
                    get { return this.frame; }
                }
                #endregion Frame

                #region GetContext
                public override object GetContext(Type type, bool checkParentElementContexts)
                {
                    if ( type == typeof(VideoFrame) )
                        return this.frame;

                    return base.GetContext(type, checkParentElementContexts);
                }
                #endregion GetContext
            }
            #endregion VideoFrameElement

            #region VideoTrackElement
            public class VideoTrackElement : UIElement
            {
                private Color borderColor = Color.Navy;
                private Color backColor = Color.LightBlue;
                private Color backColor2 = Color.White;
                private VideoTrack track = null;

                public VideoTrackElement( UIElement parent, VideoTrack track ) : base( parent )
                {
                    this.track = track;
                }

                #region BorderStyle
                public override UIElementBorderStyle BorderStyle
                {
                    get { return UIElementBorderStyle.TwoColor; }
                    set { throw new NotSupportedException(); }
                }
                #endregion BorderStyle

                #region InitAppearance
                /// <summary>
                /// Override this to apply colors and other visual formatting attributes for the element.
                /// </summary>
                protected override void InitAppearance(ref AppearanceData appearance, ref AppearancePropFlags requestedProps)
                {
                    #region BorderColor
                    if ( (requestedProps & AppearancePropFlags.BorderColor) == AppearancePropFlags.BorderColor &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor) == false )
                    {
                        appearance.BorderColor = this.borderColor;
                        requestedProps &= ~AppearancePropFlags.BorderColor;
                    }
                    #endregion BorderColor

                    #region BorderColor2
                    if ( (requestedProps & AppearancePropFlags.BorderColor2) == AppearancePropFlags.BorderColor2 &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BorderColor2) == false )
                    {
                        appearance.BorderColor2 = Color.White;
                        requestedProps &= ~AppearancePropFlags.BorderColor2;
                    }
                    #endregion BorderColor2

                    #region BackColor
                    if ( (requestedProps & AppearancePropFlags.BackColor) == AppearancePropFlags.BackColor &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BackColor) == false )
                    {
                        appearance.BackColor = this.backColor;
                        requestedProps &= ~AppearancePropFlags.BackColor;
                    }
                    #endregion BackColor

                    #region BackColor2
                    if ( (requestedProps & AppearancePropFlags.BackColor2) == AppearancePropFlags.BackColor2 &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BackColor2) == false )
                    {
                        appearance.BackColor2 = this.backColor2;
                        requestedProps &= ~AppearancePropFlags.BackColor2;
                    }
                    #endregion BackColor2

                    #region BackGradientStyle
                    if ( (requestedProps & AppearancePropFlags.BackGradientStyle) == AppearancePropFlags.BackGradientStyle &&
                         appearance.HasPropertyBeenSet(AppearancePropFlags.BackGradientStyle) == false )
                    {
                        appearance.BackGradientStyle = GradientStyle.Vertical;
                        requestedProps &= ~AppearancePropFlags.BackGradientStyle;
                    }
                    #endregion BackGradientStyle
                }
                #endregion InitAppearance

                #region Track
                public VideoTrack Track { get { return this.track; } }
                #endregion Track

                #region GetContext
                public override object GetContext(Type type, bool checkParentElementContexts)
                {
                    if ( type == typeof(VideoTrack) )
                        return this.track;

                    return base.GetContext(type, checkParentElementContexts);
                }
                #endregion GetContext

                #region OnDoubleClick
                protected override void OnDoubleClick(bool adjustableArea)
                {
                    base.OnDoubleClick(adjustableArea);
                    this.track.OnDoubleClick();
                }
                #endregion OnDoubleClick
            }
            #endregion VideoTrackElement
        }
        #endregion VideoFrameCreationFilter class

        #region VideoFrameInfo class
        public class VideoFrameInfo
        {
            private UltraTimelineView control = null;
            private DateTime zero = DateTime.Today;
            private VideoFrame currentFrame = null;

            public VideoFrameInfo( UltraTimelineView control, DateTime zero )
            {
                this.control = control;
                this.zero = zero;
            }

            #region FrameFromPoint
            /// <summary>
            /// Returns the frame number from the specified point
            /// </summary>
            /// <param name="clientPos">The point to test, expressed in client coordinates.</param>
            /// <returns>The frame number or null if the hit test was not on a frame element.</returns>
            public long? FrameFromPoint( Point clientPos )
            {
                UIElement controlElement = this.control.UIElement;
                UIElement elementAtPoint = controlElement != null ? controlElement.ElementFromPoint( clientPos ) : null;

                while ( elementAtPoint != null )
                {
                    VideoFrameCreationFilter.VideoFrameElement frameElement = elementAtPoint as VideoFrameCreationFilter.VideoFrameElement;
                    if ( frameElement != null )
                        return frameElement.Frame;

                    elementAtPoint = elementAtPoint.Parent;
                }

                return null;
            }
            #endregion FrameFromPoint

            #region CurrentFrame
            [Description("Returns information about the current video frame.")]
            [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
            public VideoFrame CurrentFrame
            {
                get
                { 
                    if ( this.currentFrame == null )
                        this.currentFrame = new VideoFrame( this.control, this.zero );

                    return this.currentFrame;
                }

            }
            #endregion CurrentFrame


        }
        #endregion VideoFrameInfo class

        #region VideoTrack
        public class VideoTrack
        {
            private long startFrame = 0;
            private long endFrame = 0;
            public VideoTrack( long startFrame, long endFrame )
            {
                this.startFrame = startFrame;
                this.endFrame = endFrame;
            }

            public long StartFrame{ get { return this.startFrame; } }
            public long EndFrame{ get { return this.endFrame; } }

            
            public delegate void DoubleClickHandler( object sender, TrackEventArgs e );
            public event DoubleClickHandler DoubleClick;

            internal protected virtual void OnDoubleClick()
            {
                if ( this.DoubleClick != null )
                    this.DoubleClick( this, new TrackEventArgs(this) );
            }

            public override string ToString()
            {
                return string.Format( "Frame[{0} - {1}]", this.startFrame, this.endFrame );
            }
        }

        public class TrackEventArgs : EventArgs
        {
            private VideoTrack track = null;
            public TrackEventArgs( VideoTrack track ){ this.track = track; }
            public VideoTrack Track { get { return  this.track; } }
        }
        #endregion VideoTrack

        #region VideoFrame class
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public class VideoFrame
        {
            private UltraTimelineView control = null;
            private DateTime zero = DateTime.Today;

            public VideoFrame( UltraTimelineView control, DateTime zero )
            {
                this.control = control;
                this.zero = zero;
            }

            #region FrameNumber
            [Description("Returns or sets the number of the current video frame (one-based).")]
            [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
            public long FrameNumber
            {
                get
                { 
                    DateTime firstVisibleTime = this.control.VisibleDateTimeRange.StartDateTime;
                    long delta = (long)(firstVisibleTime.Subtract(this.zero).TotalMinutes);
                    return delta + 1;
                }

                set
                {
                    if ( value < 1 )
                        throw new ArgumentOutOfRangeException("Can't be negative.");

                    this.control.EnsureDateTimeVisible( this.zero.Add( TimeSpan.FromMinutes(value - 1) ) );
                }
            }
            #endregion FrameNumber

            #region Hour
            /// <summary>
            /// Returns the hour of the current frame.
            /// </summary>
            public int Hour
            {
                get
                {
                    DateTime firstVisibleTime = this.control.VisibleDateTimeRange.StartDateTime;
                    long delta = (long)(firstVisibleTime.Subtract(this.zero).TotalMinutes);
                    int h = 0, m = 0, s = 0, f = 0;
                    VideoFrameTimeInterval.TimeFromFrame( delta, out h, out m, out s, out f );
                    return (int)h;
                }
            }
            #endregion Hour

            #region Minute
            /// <summary>
            /// Returns the minute of the current frame.
            /// </summary>
            public int Minute
            {
                get
                {
                    DateTime firstVisibleTime = this.control.VisibleDateTimeRange.StartDateTime;
                    long delta = (long)(firstVisibleTime.Subtract(this.zero).TotalMinutes);
                    int h = 0, m = 0, s = 0, f = 0;
                    VideoFrameTimeInterval.TimeFromFrame( delta, out h, out m, out s, out f );
                    return (int)m;
                }
            }
            #endregion Minute

            #region Second
            /// <summary>
            /// Returns the second of the current frame.
            /// </summary>
            public int Second
            {
                get
                {
                    DateTime firstVisibleTime = this.control.VisibleDateTimeRange.StartDateTime;
                    long delta = (long)(firstVisibleTime.Subtract(this.zero).TotalMinutes);
                    int h = 0, m = 0, s = 0, f = 0;
                    VideoFrameTimeInterval.TimeFromFrame( delta, out h, out m, out s, out f );
                    return (int)s;
                }
            }
            #endregion Second

            #region Frame
            /// <summary>
            /// Returns the frame.
            /// </summary>
            public int Frame
            {
                get
                {
                    DateTime firstVisibleTime = this.control.VisibleDateTimeRange.StartDateTime;
                    long delta = (long)(firstVisibleTime.Subtract(this.zero).TotalMinutes);
                    int h = 0, m = 0, s = 0, f = 0;
                    VideoFrameTimeInterval.TimeFromFrame( delta, out h, out m, out s, out f );
                    return (int)f;
                }
            }
            #endregion Frame

            #region ToString
            public override string  ToString()
            {
                 return string.Empty;
            }
            #endregion ToString
        }
        #endregion VideoFrame class
    }
    #endregion frmMain class
}
